#!/bin/bash

DIRNAME=`dirname $0`
DISKDBFILE="$DIRNAME/disk.db"
FSTABFILE=/etc/fstab
VERBOSE="Y"

# Log a string via the syslog facility.
log()
{
    if test $1 != debug || expr "$VERBOSE" : "[yY]" > /dev/null; then
        logger -p user.info -t "diskregistry[$$]" -- "$2"
    fi
}

getNextDiskNo()
{  
	log debug "getting a new disk number for $DISK"
	#If file is not there disk number will be 1
	if [ ! -f $DISKDBFILE ] ; then
		echo 1
		return
	fi
	
	#If file already has entries, find the next disk number in the sequence
	local DISKNO=0
	local lastline
	while read line
	do
		line=${line/^ .*$/}
		if [ ! -z "$line" ] ; then
			lastline="$line"
		fi
	done < $DISKDBFILE
	DISKNO=`echo "$lastline" | cut -d '|' -f2`
	DISKNO=$(( DISKNO + 1 ))
	echo $DISKNO
}

addNewEntry()
{
	log debug "adding a new entry into disk database for $DISK"
	#Backup existing file
	if [ -f $DISKDBFILE ] ; then
		cp $DISKDBFILE $DISKDBFILE~
	fi
	DISKNO=$(getNextDiskNo)
	DISKID=$(getDiskID "$@")
	DISKDETAILS=`echo $ID_VENDOR $ID_MODEL`
	DISKSIZE=$(getDiskSize "$@")
	echo "DISK|$DISKNO|$DISKID|$DISKDETAILS|$DISKSIZE" >> $DISKDBFILE
	echo $DISKNO
#	set >> $DISKDBFILE
}

deleteEntry()
{
	#Backup existing file
	if [ -f $DISKDBFILE ] ; then
		cp $DISKDBFILE $DISKDBFILE~
	fi
	
	DISK=$1
	log debug "deleting entry for $DISK from disk database"
	DISKID=$(getDiskID "$@")
	if [ -z $DISKID ] ; then 
		return
	fi
	sed -i "/^.*$DISKID.*$/d" $DISKDBFILE
}

resetRegistry()
{
	#Backup existing file
        if [ -f $DISKDBFILE ] ; then
                cp $DISKDBFILE $DISKDBFILE~
        fi
        if [ -f $FSTABFILE ] ; then
                cp $FSTABFILE $FSTABFILE~
        fi
	echo > $DISKDBFILE

	sed -i "/^UUID=.*$/d" $FSTABFILE
	exit 0
}

getDiskSize()
{
	DISK=$1
	log debug "trying to find disk size for $DISK"
	DISKSIZE=`fdisk -l /dev/$DISK | grep "Disk.*bytes$" | sed -e 's/^.*: //g' -e 's/,.*$//g'`
	echo $DISKSIZE
}

getDiskDetails()
{
	DISK=$1
	log debug "trying to find disk details for $DISK"
	DISKID=`ls -l  /dev/disk/by-id/ | grep $DISK$ | sed -e 's/^.*usb-//g' -e 's/-.*$//g'`
	DISKDETAILS=`echo ${DISKID%_*} | sed -e 's/_/ /g'`
	echo $DISKDETAILS
}

getDiskSerialNumber()
{
	DISK=$1
	log debug "trying to find disk serial from devfs for $DISK"
	DISKID=`ls -l  /dev/disk/by-id/ | grep $DISK$ | sed -e 's/^.*usb-//g' -e 's/-.*$//g'`
	DISKSERIAL=`echo ${DISKID##*_}`
	if [ -z $DISKSERIAL ]; then
		log debug "no serial number found, using the entire devfs string as ID for $DISK"
		#Use the entire path as ID
		echo $DISKID
	else
		log debug "found serial for $DISK"
		echo $DISKSERIAL
	fi
}

getDiskID()
{
	log debug "Getting disk identifier for $DISK"
	ID=`echo $ID_SERIAL_SHORT`
	if [ -z $ID ]; then
		log debug "ID not set in env, looking in the devfs $DISK"
		ID=$(getDiskSerialNumber "$1")
	fi
	log debug "Disk identifier for $DISK is $ID"
	echo $ID
}

getDiskNoFromRegistry()
{
	log debug "checking disk database for $DISK"
	if [ ! -f $DISKDBFILE ] ; then
		return
	fi
	DISK=$1
	DISKID=$(getDiskID "$@")
	DISKNO=`cat "$DISKDBFILE" | grep $DISKID | cut -d '|' -f2`
	if [ ! -z $DISKNO ]; then
		echo $DISKNO
	fi
}
getDiskNumber()
{
	DISK=$1
	log debug "finding disk number for $DISK"
	DISKNO=$(getDiskNoFromRegistry "$@")
	if [ -z $DISKNO ]; then
		log debug "no entries found in disk database for $DISK"
		DISKNO=$(addNewEntry "$@")
	fi
	echo $DISKNO
}

COMMAND=$1
log $COMMAND
shift

if [ "$COMMAND" = "getdiskdetails" ] ; then
	val=$(getDiskDetails "$@")
	echo $val
elif [ "$COMMAND" = "delentry" ] ; then
	deleteEntry "$@"
elif [ "$COMMAND" = "clear" ] ; then
	resetRegistry "$@"
elif [ "$COMMAND" = "getdisknumber" ] ; then
	id=$(getDiskNumber "$@")
	echo $id
fi
