#! /bin/bash

DIRNAME=`dirname $0`
WPACONFIG="$DIRNAME/current_wpa.conf"
WPALOG="$DIRNAME/wpa_supplicant.log"
DEBUGLOG="$DIRNAME/wireless.log"
MODEFILE="$DIRNAME/current.state"
UAPUTL="/usr/sbin/uaputl"

source $DIRNAME/default.properties

# Return xml message
echoMessage()
{
	echo "<messages>"
	echo "<message>"
	echo $1
	echo "<message>"
	echo "</messages>"
}

# Log message
echoLog()
{
	TIMESTAMP=`date +"%b %d %T : "`
	echo "$TIMESTAMP"$1 >> $DEBUGLOG
#	echo "$TIMESTAMP"$1
}

# Sets whether the wireless is a client or access point
setWirelessMode()
{
	echoLog "Trying to create wireless access point"	
	if [[ -f $MODEFILE ]]; then
		MODE=`cat $MODEFILE | grep MODE=`
	fi  
	if [[ -z $MODE ]]; then
  		echo "MODE=$1" >> $MODEFILE 
	else
  		sed -i  "s/^MODE=.*$/MODE=$1/g" $MODEFILE  
  	fi  

}

selectAPDHCPSetting()
{
	echoLog "Setting DHCP Range"	
	APNO=1
	if [[ ! -z $1 ]]; then
		APNO=$1
	fi
	
	if [[ -f $MODEFILE ]]; then
		APINFONO=`cat $MODEFILE | grep APINFONO=`
	fi  
	if [[ -z $APINFONO ]]; then
  		echo "APINFONO=$APNO" >> $MODEFILE 
	else
  		sed -i  "s/^APINFONO=.*$/APINFONO=$APNO/g" $MODEFILE  
  	fi  
}

# Switch wireless to stored state
# Called when the system boots up
switchWirelessState()
{
	echoLog "Switching wireless state"
	MODE=disabled
	if [[ -f $MODEFILE ]]; then
		eval `cat $MODEFILE | grep MODE`
	fi
	echoLog "Setting wireless to $MODE state"
	case "$MODE" in
  		client)
  			enableWirelessDriver
       		connectClient
        	;;
		accesspoint)
			enableWirelessDriver
        	startAccessPoint
        	;;
		*)
        	disableWirelessDriver
	        ;;
	esac

}

disableWirelessDriver()
{
	disconnectClient
	echoLog "Disabling wireless driver $WIRELESS_DRIVER"
	/sbin/modprobe -r $WIRELESS_DRIVER 	
}

enableWirelessDriver()
{
	echoLog "Enabling wireless driver $WIRELESS_DRIVER"
	/sbin/modprobe $WIRELESS_DRIVER 
}

startDHCPDNSService()
{
	if [[ -f $MODEFILE ]]; then
		APINFONO="`cat $MODEFILE | grep APINFONO | sed -e 's/^.*APINFONO=//g'`"
	fi  
	
	if [[ -z $APINFONO ]]; then
		echoLog "Access point dhcp range not found. Using defaults..."
		APINFONO=1
	fi
	
	APIP=`echo ${APSUBNET[$APINFONO]} | cut -d '|' -f 1`
	APDHCPINFO=`echo ${APSUBNET[$APINFONO]} | cut -d '|' -f 2`

	if [[ -z $APIP ]]; then
		echoLog "Access point address info missing. Please check default.properties file"
  		exit 1
	else
		echoLog "Setting IP address of uap0 to $APIP"
  		ifconfig uap0 $APIP
  	fi  
  	
  	# Create dhcp configuration
  	echo "interface=uap0" > /etc/dnsmasq.d/tonidoplug
  	if [[ -z $APDHCPINFO ]]; then
		echoLog "Access point DHCP info missing. Please check default.properties file"
	else
		echoLog "Setting DHCP info to $APDHCPINFO"
  		echo "dhcp-range=$APDHCPINFO" >> /etc/dnsmasq.d/tonidoplug
  	fi  
  	echo "domain=$APDOMAIN" >> /etc/dnsmasq.d/tonidoplug
	echo "address=$DNSOPTION" >> /etc/dnsmasq.d/tonidoplug
	
	/etc/init.d/dnsmasq stop
	/etc/init.d/dnsmasq start
}

configAccessPoint()
{
	echoLog "Configuring wireless access point"	

	echoLog "Stopping ssid broadcast"
	$UAPUTL bss_stop
	
	#Set SSID
	if [[ -f $WPACONFIG ]]; then
		SSID="`cat $WPACONFIG | grep ssid | sed -e 's/^.*ssid=\"//g' -e 's/\"$//g'`"
	fi

	if [[ -z $SSID ]]; then
		echoLog "Setting SSID to $DEFAULTAPSSID"
  		$UAPUTL sys_cfg_ssid $DEFAULTAPSSID
	else
		echoLog "Setting SSID to $SSID"
  		$UAPUTL sys_cfg_ssid $SSID
  	fi

  	# Set the protocol PLAIN/WEP/WPA/WPA2 and passphrase
  	if [[ -f $WPACONFIG ]]; then
		PROTO=`cat $WPACONFIG | grep proto | sed -e 's/^.*proto=//g'`
		PASSPHRASE=`cat $WPACONFIG | grep \#psk | sed -e 's/^.*#psk=//g'`
	fi
	
	echoLog "Switching protocol to $PROTO"
	case "$PROTO" in
    	WEP)
        	$UAPUTL sys_cfg_protocol 2
        	$UAPUTL sys_cfg_wep_key 0 1 $PASSPHRASE
			;;
		WPA)
        	$UAPUTL sys_cfg_protocol 8
        	$UAPUTL sys_cfg_wpa_passphrase $PASSPHRASE
            ;;
		WPA2)
        	$UAPUTL sys_cfg_protocol 32
        	$UAPUTL sys_cfg_wpa_passphrase $PASSPHRASE
			;;
		*)
        	$UAPUTL sys_cfg_protocol 1
            ;;
	esac
	
	# Set the encryption
	if [[ -f $WPACONFIG ]]; then
		PAIRWISE="`cat $WPACONFIG | grep pairwise | sed -e 's/^.*pairwise=//g'`"
	fi
	
	echoLog "Switching encryption to $PAIRWISE"
	case "$PAIRWISE" in
    	TKIP)
		if [[ "$PROTO" = "WPA" ]]; then
			$UAPUTL sys_cfg_cipher 8 8
		else
			$UAPUTL sys_cfg_cipher 4 4
		fi
			;;
		CCMP)
        	$UAPUTL sys_cfg_cipher 8 8
            ;;
		"CCMP TKIP")
		if [[ "$PROTO" = "WPA" ]]; then
                        $UAPUTL sys_cfg_cipher 8 8
                else
                        $UAPUTL sys_cfg_cipher 12 4
                fi
			;;
		*)
        	$UAPUTL sys_cfg_cipher 0 0
            ;;
	esac

	echoLog "Starting ssid broadcast"
	$UAPUTL bss_start
}

startAccessPoint()
{
	DRIVERLOADED=`lsmod | grep $WIRELESS_DRIVER | wc -l`
	if [[ $DRIVERLOADED = "0" ]]; then
		echoLog "Driver disabled. Please enable wireless driver"
		return
	fi
	
	# Set and enable wireless params
	configAccessPoint
	
	# Start dhcp and dns service
	startDHCPDNSService
	
	# Set forwarding
	iptables -t nat -A POSTROUTING -o eth0 -j MASQUERADE
	echo 1 > /proc/sys/net/ipv4/ip_forward

}



stopAccessPoint()
{
	echoLog "Trying to stop access point"
	echoLog "Stopping ssid broadcast"
	$UAPUTL bss_stop
	
	echoLog "Stopping dns & dhcp server"
	/etc/init.d/dnsmasq stop
	pkill dnsmasq
	
	/sbin/ifconfig uap0 down
}

resetToFactorySettings()
{
	echoLog "Resetting plug wireless to factory settings"
	$DIRNAME/createWirelessInfo.sh PLAIN $DEFAULTAPSSID 
}

connectClient()
{
	DRIVERLOADED=`lsmod | grep $WIRELESS_DRIVER | wc -l`
	if [[ $DRIVERLOADED = "0" ]]; then
		echoLog "Driver disabled. Please enable wireless driver"
		return
	fi
	
	echoLog "Trying to connect to wireless access point"	
	if [[ -z $WPACONFIG ]]; then
		echoLog "Missing wireless configuration file"
		return
	fi
	
	if [[ -f $MODEFILE ]]; then
		MODE=`cat $MODEFILE | grep MODE=`
	fi  
	if [[ -z $MODE ]]; then
  		echo "MODE=client" >> $MODEFILE 
	else
  		sed -i  "s/^MODE=.*$/MODE=client/g" $MODEFILE  
  	fi  

	echo > $WPALOG
	echoLog "Client connection log stored in $WPALOG"	
	wpa_supplicant -i$INTERFACE -c $WPACONFIG -f $WPALOG -B  >& /dev/null
	waitForConnection
}

waitForConnection()
{
    echoLog "Waiting for connection to complete"    
    let timeout=TIMEOUT
    while [[ $timeout -gt 0 ]]; do
    	sleep 1
    	echoLog "Waiting for connection..."
        eval `wpa_cli status|grep wpa_state` 
        if [[ "$wpa_state" = "COMPLETED" ]]; then
        	echoLog "Success, connection completed"
			if [[ -f $MODEFILE ]]; then
				eval `cat $MODEFILE | grep ADDRESSTYPE`
				eval `cat $MODEFILE | grep ADDRESS`
				eval `cat $MODEFILE | grep NETMASK`
				eval `cat $MODEFILE | grep GATEWAY`
			fi  	
			if [[ $ADDRESSTYPE = "dhcp" ]]; then
				echoLog "Address type set to dhcp"
				requestDHCPAddress
			elif [[ $ADDRESSTYPE = "static" ]]; then
				echoLog "Address type set to static"
				setStaticAddress $ADDRESS $NETMASK $GATEWAY
			fi
        	
        	return
        fi
        
        let timeout--
    done

    wpa_cli terminate >/dev/null 2>&1
    echoLog "Connection attempt failed. Disabling wireless"
    stopAllWirelessActivity
    setWirelessMode "disabled"
    disableWirelessDriver

    exit 1
}

requestDHCPAddress()
{
	echoLog "Trying to get DHCP Address"
	HOST=`cat /etc/hostname`
	DHCPOUTPUT=`udhcpc -i mlan0 -t 10 -n -q -C -h $HOST`
	echoLog "$DHCPOUTPUT"		
}

setStaticAddress()
{
	echoLog "Setting Static Address: $1/$2"
	ifconfig $INTERFACE $1 netmask $2
	if [[ ! -z $3 ]]; then
		route add default gw $3 $INTERFACE	
	fi
}

disconnectClient()
{
	echoLog "Trying to disconnect wireless client"
	echoLog "Stopping dhcp client"
	pkill udhcpc
	echoLog "Stopping wireless client"
	wpa_cli terminate >/dev/null 2>&1
	pkill wpa_supplicant
	
	if [[ ! -S /var/run/wpa_supplicant/$INTERFACE ]]; then
        echoLog "Client disconnect success"
	else
        echoLog "Client disconnect failed"
	fi
}

stopAllWirelessActivity()
{
	echoLog "-------Stopping all wireless activity-------"
	disconnectClient
	stopAccessPoint
}

case "$1" in
  start)
  		echoLog "******************System startup******************"
  		switchWirelessState
  		;;
  stop)
  		echoLog "******************System shutdown*****************"
  		stopAllWirelessActivity
  		disableWirelessDriver
  		;;
  clientDHCP)
  		echoLog "-------Received wireless client connect request-------"
		echoLog "Disconnecting current connection"
  		stopAllWirelessActivity
  		setWirelessMode "client"
  		if [[ -f $MODEFILE ]]; then
				ADDRESSTYPE=`cat $MODEFILE | grep ADDRESSTYPE=`
		fi  
		if [[ -z $ADDRESSTYPE ]]; then
  			echo "ADDRESSTYPE=dhcp" >> $MODEFILE 
  		else
  		 	sed -i  "s/^ADDRESSTYPE=.*$/ADDRESSTYPE=dhcp/g" $MODEFILE 
  		fi  		
  		enableWirelessDriver
       	connectClient
        ;;
  clientStatic)
  		echoLog "-------Received wireless client connect request-------"
  		[[ -z $2 ]] || [[ $2 = "-" ]] && echoLog "Missing IP address" && exit
  		[[ -z $3 ]] || [[ $3 = "-" ]] && echoLog "Missing Netmask" && exit
  		[[ -z $4 ]] || [[ $4 = "-" ]] && echoLog "Missing Gateway" && exit
  		echoLog "Disconnecting current connection"
  		stopAllWirelessActivity
  		setWirelessMode "client"
  		if [[ -f $MODEFILE ]]; then
				ADDRESSTYPE=`cat $MODEFILE | grep ADDRESSTYPE=`
				ADDRESS=`cat $MODEFILE | grep ADDRESS=`
				NETMASK=`cat $MODEFILE | grep NETMASK=`
				GATEWAY=`cat $MODEFILE | grep GATEWAY=`
		fi  	
  		if [[ -z $ADDRESSTYPE ]]; then
  			echo "ADDRESSTYPE=static" >> $MODEFILE 
  		else
  		 	sed -i  "s/^ADDRESSTYPE=.*$/ADDRESSTYPE=static/g" $MODEFILE 
  		fi
  		if [[ -z $ADDRESS ]]; then
  			echo "ADDRESS=$2" >> $MODEFILE 
  		else
  			sed -i  "s/^ADDRESS=.*$/ADDRESS=$2/g" $MODEFILE 
  		fi
  		if [[ -z $NETMASK ]]; then
  			echo "NETMASK=$3" >> $MODEFILE 
  		else
  			sed -i  "s/^NETMASK=.*$/NETMASK=$3/g" $MODEFILE 
  		fi
  		if [[ -z $GATEWAY ]]; then
  			echo "GATEWAY=$4" >> $MODEFILE 
  		else
  			sed -i  "s/^GATEWAY=.*$/GATEWAY=$4/g" $MODEFILE 
  		fi
		enableWirelessDriver
       	connectClient
        ;;
  clientDisconnect)
  		echoLog "-------Received wireless client disconnect request-------"
        disconnectClient
	stopAllWirelessActivity
                setWirelessMode "disabled"
                disableWirelessDriver

        ;;
  accessPointEnable)
  		echoLog "-------Received create wireless access point request-------"
  		stopAllWirelessActivity
  		setWirelessMode "accesspoint" 
  		selectAPDHCPSetting "$2"  		
  		enableWirelessDriver
  		startAccessPoint
  		;;
  accessPointDisable)
  		echoLog "-------Received stop wireless access point request-------"
  		stopAccessPoint
		stopAllWirelessActivity
                setWirelessMode "disabled"
                disableWirelessDriver
  		;;
  reset)
  		echoLog "-------Received reset wireless to factory settings-------"
  		resetToFactorySettings
  		;;
  enable)
  		echoLog "-------Received enable wireless request--------"
        enableWirelessDriver
        ;;
  disable)
  		echoLog "-------Received disable wireless request-------"
  		stopAllWirelessActivity
  		setWirelessMode "disabled"
  		disableWirelessDriver
  		;;      
  *)
        echo "Usage: $0 {clientDHCP|clientStatic <ip> <netmask>|clientDisconnect|accessPointEnable|accessPointDisable|enable|disable}" >&2
        exit 1
        ;;
esac

exit 0

